{ $RCSfile: FwinKeyLockMain.pas,v $ $Revision: 1.9 $ $Date: 2004/03/22 05:27:35 $ $Author: vlads $ }
{                                                                                    }
{ ================================================================================== }
{                                                                                    }
{   This file is part of winKeyLock,  http://winKeyLock.sourceforge.net/             }
{   See readme.txt and license.txt for details.                                      }
{   (c) Copyright 2004 - Vlad Skarzhevskyy & co                                      }
{                                                                                    }
{   This program is free software; you can redistribute it and/or modify it under    }
{  the terms of the GNU General Public License as published by the Free Software     }
{  Foundation; either version 2 of the License, or (at your option) any later        }
{  version.                                                                          }
{                                                                                    }
{   This program is distributed in the hope that it will be useful, but WITHOUT ANY  }
{  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   }
{  PARTICULAR PURPOSE.  See the GNU General Public License for more details.         }
{                                                                                    }
{  The GNU GPL can be found at:       http://www.gnu.org/copyleft/gpl.html           }
{                                                                                    }
{ ================================================================================== }

unit FwinKeyLockMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, Menus, ImgList,
  { 3p-units }
  XPMenu,
  JvTrayIcon,
  Properties, StdCtrls, ExtCtrls;

type
  THookDLLFunc = function (): BOOL; cdecl;
  THookDLLSetNotifyFunc = function (HookNotifyWnd: HWND): BOOL; cdecl;
  THookDLLGetNotifyFunc = function (): HWND; cdecl;
  THookDLLSetPropertiesFunc = function (id: integer; value: longint): BOOL; cdecl;
  THookDLLSetStrFunc = function (str: PChar): BOOL; cdecl;

  TBalloonHintMessages = (bhmNone, bhmRunning, bhmLocked, bhmUnLocked);

  TFormWinKeyLockMain = class(TForm)
    pmTrayIcon: TPopupMenu;
    ActionList: TActionList;
    acOptions: TAction;
    acExit: TAction;
    acLock: TAction;
    pmiLockComputer: TMenuItem;
    pmiN1: TMenuItem;
    pmiOptions: TMenuItem;
    pmiN2: TMenuItem;
    pmiExit: TMenuItem;
    ImageList: TImageList;
    acUnlock: TAction;
    pmiUnlock: TMenuItem;
    acScreenSaver: TAction;
    pmiRunScreenSaver: TMenuItem;
    TimerTaskMgrHide: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure acExitExecute(Sender: TObject);
    procedure acOptionsExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure acUnlockExecute(Sender: TObject);
    procedure acUnlockUpdate(Sender: TObject);
    procedure acLockUpdate(Sender: TObject);
    procedure acScreenSaverExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure acLockExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerTaskMgrHideTimer(Sender: TObject);
  protected
    procedure CreateParams(Var Params: TCreateParams); override;
  private
    { Private declarations }
    TrayIcon: TJvTrayIcon;
    BalloonVisible: boolean;
    BalloonMsg: TBalloonHintMessages;

    XPMenu: TXPMenu;

    hHookDLLInst: HMODULE;
    SetShellHook: THookDLLFunc;
    UnSetShellHook: THookDLLFunc;
    GetIsLocked: THookDLLFunc;
    SetNotifyWnd: THookDLLSetNotifyFunc;
    GetNotifyWnd: THookDLLGetNotifyFunc;
    SetProperties: THookDLLSetPropertiesFunc;
    StartLog: THookDLLSetStrFunc;
  public
    { Public declarations }
    procedure TrayIconClick(Sender: TObject; Button: TMouseButton;
                            Shift: TShiftState; X, Y: Integer);

    procedure TrayIconDblClick(Sender: TObject; Button: TMouseButton;
                                 Shift: TShiftState; X, Y: Integer);
    procedure TrayIconOnBalloonClick(Sender: TObject);
    function LoadShellHookDLL(ShowErrors: boolean): boolean;

    procedure wmUSER(var msg: TMessage); message WM_USER;
    procedure UpdateStatus();
    procedure ShowLocked(WParam: longint = 0);
    procedure ShowUnLocked();
    procedure ShowIsRunning();
    procedure LoadArguments();
    class procedure ShowCommandLineOptions(hw: HWND);
    procedure SetTrayIcon(locked: boolean);
    procedure setLock(lock: boolean);
    procedure UpdateLockSettings();
    procedure AfterInitialization();
    procedure BalloonHintShow(messageID: TBalloonHintMessages);
    procedure OnBalloonHide(Sender: TObject);
    procedure OnBalloonShow(Sender: TObject);
    procedure HideTaskMgr();
  end;

var
    FormWinKeyLockMain: TFormWinKeyLockMain;
    hMutex: Cardinal;
    isMainProcess: boolean;

const
    sMutex = 'WinKeyLockMutex';
    sHookDll = 'winKeyLockHook.dll';

implementation

{$R *.dfm}

uses
    FOptions, FPopup;

procedure TFormWinKeyLockMain.FormCreate(Sender: TObject);
begin
    hHookDLLInst := 0;
    TrayIcon := nil;
    IniFile := TLockIniFile.Create(Application.ExeName);
    IniFile.Read;

    LoadArguments();

    if (NOT isMainProcess) then Exit;
    BalloonVisible:= false;
    BalloonMsg := bhmNone;

    TrayIcon := TJvTrayIcon.Create(self);
    TrayIcon.Active := false;
    TrayIcon.OnDblClick := TrayIconDblClick;
    TrayIcon.OnClick := TrayIconClick;
    TrayIcon.OnBalloonClick := TrayIconOnBalloonClick;
    TrayIcon.OnBalloonShow := OnBalloonShow;
    TrayIcon.OnBalloonHide := OnBalloonHide;

    TrayIcon.PopupMenu := pmTrayIcon;
    TrayIcon.Active := true;
    SetTrayIcon(false);

    XPMenu := TXPMenu.Create(self);
    XPMenu.Form := self;
    XPMenu.DimLevel := 30;
    XPMenu.GrayLevel := 10;
    XPMenu.Color := clBtnFace;
    XPMenu.IconBackColor := clBtnFace;
    XPMenu.MenuBarColor := clBtnFace;
    XPMenu.SelectColor := clHighlight;
    XPMenu.SelectBorderColor := clHighlight;
    XPMenu.SelectFontColor := clMenuText;
    XPMenu.DisabledColor := clInactiveCaptionText;
    XPMenu.SeparatorColor := clBtnFace;
    XPMenu.CheckedColor := clHighlight;
    XPMenu.DrawSelect := True;
    XPMenu.UseSystemColors := True;
    XPMenu.OverrideOwnerDraw := True;
    XPMenu.XPControls := [xcMainMenu, xcToolbar, xcPopupMenu];
    XPMenu.XPContainers := [xccControlbar, xccToolbar];
    XPMenu.FlatMenu := False;
    XPMenu.AutoDetect := True;
    XPMenu.DisableSubclassing := false;
    XPMenuManager.Add(XPMenu);
    xpMenu.Active := true;

    UpdateLockSettings;

    // To prevent the window button from being placed on the taskbar
    SetWindowLong(Application.Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
    PostMessage(self.Handle, WM_USER, 0, WKL_CMD_INITIALIZED);
end;

procedure TFormWinKeyLockMain.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.Style := Params.Style and not WS_CAPTION or WS_POPUP;
  Params.ExStyle := Params.ExStyle or WS_EX_TOOLWINDOW or WS_EX_NOACTIVATE;
end;

procedure TFormWinKeyLockMain.AfterInitialization;
begin
    if (IniFile.isFirstRun) then
        acOptions.Execute()
    else
        ShowIsRunning();
end;

procedure TFormWinKeyLockMain.FormDestroy(Sender: TObject);
begin
    IniFile.Free;
    TrayIcon.Free;
    XPMenu.Free;
end;

procedure TFormWinKeyLockMain.FormShow(Sender: TObject);
begin
    if (TrayIcon <> nil) then TrayIcon.HideApplication;
end;

procedure TFormWinKeyLockMain.FormClose(Sender: TObject;
                                        var Action: TCloseAction);
begin
    if (hHookDLLInst <> 0) then
    begin
        if (isMainProcess) then
        begin
            setLock(false);
            UnSetShellHook();
            StartLog(nil);
        end;
        FreeLibrary(hHookDLLInst);
        hHookDLLInst := 0;
    end;
end;

procedure TFormWinKeyLockMain.TrayIconClick(Sender: TObject;
                    Button: TMouseButton;
                    Shift: TShiftState;
                    X, Y: Integer);
begin
    if (Button = mbRight) then Exit;
    acOptions.Execute();
end;

procedure TFormWinKeyLockMain.TrayIconDblClick(Sender: TObject;
                    Button: TMouseButton;
                    Shift: TShiftState;
                    X, Y: Integer);
begin
    acLock.Execute();
end;

procedure TFormWinKeyLockMain.TrayIconOnBalloonClick(Sender: TObject);
begin

end;

procedure TFormWinKeyLockMain.acExitExecute(Sender: TObject);
begin
    Close;
end;

procedure TFormWinKeyLockMain.acOptionsExecute(Sender: TObject);
var
    mr: TModalResult;
begin
    ForceForeground(Application.Handle);
    if (FormOptions <> nil) then Exit;

    SetWindowLong(Application.Handle, GWL_EXSTYLE, 0);
    Application.ProcessMessages;

    FormOptions := TFormOptions.Create(Application);

    //  Disable Keys temporary
    SetProperties(WKL_PROP_KEY_LOCK, 0);
    SetProperties(WKL_PROP_KEY_UNLOCK, 0);
    acLock.ShortCut := 0;
    acUnLock.ShortCut := 0;

    mr := FormOptions.ShowModal();
    UpdateLockSettings;
    if LoadShellHookDLL(false) then
        setLock(GetIsLocked());

    if (IniFile.isFirstRun) then
        ShowIsRunning();

    IniFile.Write();

    FormOptions := nil;
    if (mr = mrYes) then
       acLock.Execute;
    //if (TrayIcon <> nil) then TrayIcon.HideApplication;
    SetWindowLong(Application.Handle, GWL_EXSTYLE, WS_EX_TOOLWINDOW);
end;

function TFormWinKeyLockMain.LoadShellHookDLL(ShowErrors: boolean): boolean;
var
    FPointer: TFarProc;
    DLLFilePath : string;
    LOGFilePath : string;
const
    buildPath =  'HookDll\obj\';
    procedure DoShowError(ErrorMessage: String);
    begin
        if (ShowErrors) then
            MessageDlg(ErrorMessage,
                   mtError, [mbOK], 0);
        raise Exception.Create(ErrorMessage);
    end;
begin
    Result := true;
    if (hHookDLLInst <> 0) then Exit;

    try
        DLLFilePath := ExtractFilePath(ParamStr(0)) + sHookDll;
        if NOT FileExists(DLLFilePath) then
        begin
            // Mp3BH Build
            DLLFilePath := ExtractFilePath(ParamStr(0)) + buildPath + sHookDll;
            if NOT FileExists(DLLFilePath) then
                // Let Windows find it.
                DLLFilePath := sHookDll;
        end;

        hHookDLLInst := SafeLoadLibrary(DLLFilePath);
        if (hHookDLLInst = 0) then DoShowError('Dll:' + sHookDll + ' not found');

        LOGFilePath := ChangeFileExt(DLLFilePath,'.log');

        FPointer := GetProcAddress(hHookDLLInst, 'SetShellHook');
        if FPointer = nil then DoShowError(sHookDll + '.SetShellHook DLL function not found');
        SetShellHook := THookDLLFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'UnSetShellHook');
        if FPointer = nil then DoShowError(sHookDll + '.UnSetShellHook DLL function not found');
        UnSetShellHook := THookDLLFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'SetNotifyWnd');
        if FPointer = nil then DoShowError(sHookDll + '.SetNotifyWnd DLL function not found');
        SetNotifyWnd := THookDLLSetNotifyFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'GetIsLocked');
        if FPointer = nil then DoShowError(sHookDll + '.GetIsLocked DLL function not found');
        GetIsLocked := THookDLLFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'GetNotifyWnd');
        if FPointer = nil then DoShowError(sHookDll + '.GetNotifyWnd DLL function not found');
        GetNotifyWnd := THookDLLGetNotifyFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'SetProperties');
        if FPointer = nil then DoShowError(sHookDll + '.SetProperties DLL function not found');
        SetProperties := THookDLLSetPropertiesFunc(FPointer);

        FPointer := GetProcAddress(hHookDLLInst, 'StartLog');
        if FPointer = nil then DoShowError(sHookDll + '.StartLog DLL function not found');
        StartLog := THookDLLSetStrFunc(FPointer);

        if (isMainProcess) then
        begin
            SetNotifyWnd(Handle);
            //StartLog(PChar(LOGFilePath + #0));
        end;

        Result := true;
    except
        FreeLibrary(hHookDLLInst);
        hHookDLLInst := 0;
        Result := false;
    end;
end;

procedure TFormWinKeyLockMain.setLock(lock: boolean);
var
    flag: longint;
begin
    if (LoadShellHookDLL(true)) then
    begin
        if (isMainProcess) then
        begin
            UpdateLockSettings;

            flag := 0;
            if (lock) then flag := 1;

            if (NOT SetProperties(WKL_PROP_LOCKED, flag)) then
                acOptions.Execute();
        end
        else
        begin
            if (lock) then
                SendMessage(GetNotifyWnd(), WM_USER, 0, WKL_CMD_LOCK)
            else
                SendMessage(GetNotifyWnd(), WM_USER, 0, WKL_CMD_UNLOCK);
        end;
    end;
end;

procedure TFormWinKeyLockMain.UpdateLockSettings;
var
    i: integer;
begin
    if (LoadShellHookDLL(true)) and (isMainProcess) then
    begin
        for i := 1 to WKL_PROP_MAX-1 do
        begin
            SetProperties(i, IniFile.LockSettings.values[i]);
        end;
        SetShellHook();
    end;
    acLock.ShortCut :=  IniFile.LockSettings.values[WKL_PROP_KEY_LOCK];
    acUnLock.ShortCut :=  IniFile.LockSettings.values[WKL_PROP_KEY_UNLOCK];

    acLock.ImageIndex := IniFile.LockSettings.values[WKL_PROP_LOKED_IDX];
    acUnLock.ImageIndex := IniFile.LockSettings.values[WKL_PROP_UNLOKED_IDX];
end;

procedure TFormWinKeyLockMain.acLockExecute(Sender: TObject);
begin
    setLock(true);
end;

procedure TFormWinKeyLockMain.acUnlockExecute(Sender: TObject);
begin
    setLock(false);
end;

procedure TFormWinKeyLockMain.acUnlockUpdate(Sender: TObject);
begin
    acUnlock.Enabled := (hHookDLLInst <> 0) and GetIsLocked();
end;

procedure TFormWinKeyLockMain.acLockUpdate(Sender: TObject);
begin
    acLock.Enabled := (hHookDLLInst = 0) or
                      ((hHookDLLInst <> 0) and (NOT GetIsLocked()));
end;

procedure TFormWinKeyLockMain.wmUSER(var msg: TMessage);
begin
    case msg.LParam of
        WKL_CMD_INITIALIZED: AfterInitialization();
        WKL_CMD_VERSION: ReplyMessage(1);
        WKL_CMD_UPDATE_STATUS: UpdateStatus();
        WKL_CMD_SHOW_LOCKED: ShowLocked(msg.WParam);
        WKL_CMD_LOCK:
            begin
                acLockExecute(nil);
                ReplyMessage(WKL_CMD_LOCK);
            end;
        WKL_CMD_UNLOCK:
            begin
                acUnLockExecute(nil);
                ReplyMessage(WKL_CMD_UNLOCK);
            end;
        WKL_CMD_HIDE_TASKMGR: HideTaskMgr();

    end;
end;

procedure TFormWinKeyLockMain.OnBalloonHide(Sender: TObject);
begin
    BalloonVisible := false;
    BalloonMsg := bhmNone;
end;

procedure TFormWinKeyLockMain.OnBalloonShow(Sender: TObject);
begin
    BalloonVisible := true;
end;

procedure TFormWinKeyLockMain.BalloonHintShow(messageID: TBalloonHintMessages);
var
    sTitle: String;
    sMessage : String;
    //blType: TBalloonType;
begin
    if (TrayIcon = nil) then Exit;
    //if (BalloonVisible and (BalloonMsg = messageID)) then Exit;
    case messageID of
        bhmRunning:
            begin
                sTitle := 'winKeyLock';
                sMessage := 'Program is running';
                //blType := btInfo;
            end;
        bhmLocked:
            begin
                sTitle := 'Computer is Locked';
                sMessage := 'Enter Hot Key or password to Unlock';
                //blType := btWarning;
            end;
        bhmUnLocked:
            begin
                sTitle := 'Computer is UnLocked';
                sMessage := 'You may continue to work.';
                //sTitle := '';
                //sMessage := '';
                //blType := btInfo;
            end;
        else
            begin
                sTitle := '';
                sMessage := '';
                //blType := btInfo;
            end;
    end;
    BalloonMsg := messageID;

    {
    Hive: HKEY_CURRENT_USER
    Key: Software\Microsoft\Windows\CurrentVersion\Explorer\Advanced
    Name: EnableBalloonTips
    Type: REG_DWORD
    }

    //TrayIcon.BalloonHint(sTitle, sMessage, blType,
    //                     5000, true);

    TFormPopup.BalloonHint(sTitle, sMessage, 5000, TrayIcon.Icon);
end;

procedure TFormWinKeyLockMain.ShowLocked(WParam: longint);
begin
    BalloonHintShow(bhmLocked);
end;

procedure TFormWinKeyLockMain.ShowUnLocked;
begin
    BalloonHintShow(bhmUnLocked);
end;

procedure TFormWinKeyLockMain.ShowIsRunning();
begin
    BalloonHintShow(bhmRunning);
end;

procedure TFormWinKeyLockMain.UpdateStatus;
var
    locked: boolean;
begin
    if (LoadShellHookDLL(false)) then
    begin
        acUnlockUpdate(nil);
        acLockUpdate(nil);
        locked := GetIsLocked();
        SetTrayIcon(locked);

        if (locked) then
            ShowLocked()
        else
            ShowUnLocked();
    end;
end;

procedure TFormWinKeyLockMain.SetTrayIcon(locked: boolean);
var
    idx: integer;
begin
    if locked then
        idx := acLock.ImageIndex
    else
        idx := acUnLock.ImageIndex;

    ImageList.GetIcon(idx, TrayIcon.Icon);
end;

procedure TFormWinKeyLockMain.acScreenSaverExecute(Sender: TObject);
begin
    //LockWorkStation();
    SendMessage(HWND_TOPMOST, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
end;

procedure TFormWinKeyLockMain.HideTaskMgr();
var
    Wnd: HWND;
begin
    Wnd := FindWindow(nil, 'Task Manager');
    if (Wnd = 0) then
        Wnd := FindWindow(nil, 'Windows Task Manager');

    if (Wnd <> 0) then
    begin
        PostMessage(Wnd, WM_CLOSE, 0, 0);
        TimerTaskMgrHide.Enabled := false;
    end
    else
        TimerTaskMgrHide.Enabled := true;
end;

procedure TFormWinKeyLockMain.TimerTaskMgrHideTimer(Sender: TObject);
begin
    HideTaskMgr();
end;

function isArgSwitch(s : String) : Boolean;
begin
   Result := false;
   if (Length(s) <> 2) then Exit;
   if s[1] = '-' then Result := true;
   if s[1] = '/' then Result := true;
end;

class procedure TFormWinKeyLockMain.ShowCommandLineOptions(hw: HWND);
var
    Desc : string;
begin
    Desc :=
ExtractFileName(ParamStr(0)) + ' [/L|/U]' + sLineBreak +
'   /L     Lock Computer'  + sLineBreak +
'   /U     UnLock Computer '  + sLineBreak;

    MessageBox(hw, PChar(Desc),
           'WinKeyLock Command-Line Options', mb_IconInformation or mb_OK );
end;

procedure TFormWinKeyLockMain.LoadArguments;
var
    i: integer;
    p: string;
begin
    for i := 1 to ParamCount do
    begin
        p := LowerCase(ParamStr(i));
        if isArgSwitch(p) then
        begin
            if (p[2] = 'l') then acLockExecute(nil);
            if (p[2] = 'u') then acUnLockExecute(nil);
            if (p[2] in [Char('h'), Char('H'), Char('?') ]) then
                 ShowCommandLineOptions(Handle);
            Continue;
        end;
    end;
    if (NOT isMainProcess) and (ParamCount = 0) and (LoadShellHookDLL(true)) then
    begin
        PostMessage(GetNotifyWnd(), WM_USER, 0, WKL_CMD_INITIALIZED);
    end;

    if (NOT isMainProcess) then
        PostMessage(Handle, WM_CLOSE, 0, 0);
end;

end.
