/* $RCSfile: winKeyLockHook.c,v $ $Revision: 1.5 $ $Date: 2004/03/22 04:17:41 $ $Author: vlads $ */
/*                                                                                    */
/* ================================================================================== */
/*                                                                                    */
/*   This file is part of winKeyLock,  http://winKeyLock.sourceforge.net/             */
/*   See readme.txt and license.txt for details.                                      */
/*   (c) Copyright 2004 - Vlad Skarzhevskyy & co                                      */
/*                                                                                    */
/*   This program is free software; you can redistribute it and/or modify it under    */
/*  the terms of the GNU General Public License as published by the Free Software     */
/*  Foundation; either version 2 of the License, or (at your option) any later        */
/*  version.                                                                          */
/*                                                                                    */
/*   This program is distributed in the hope that it will be useful, but WITHOUT ANY  */
/*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/*  PARTICULAR PURPOSE.  See the GNU General Public License for more details.         */
/*                                                                                    */
/*  The GNU GPL can be found at:       http://www.gnu.org/copyleft/gpl.html           */
/*                                                                                    */
/* ================================================================================== */

// winKeyLockHook.c : Defines the implementation of the DLL.
//

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0501 // for Constants
#endif

#include <tchar.h>
#include <windows.h>
#include <winuser.h>
#include <winreg.h>
#include "winKeyLockHook.h"
#include "win_mesages.inc"


//#include <crtdbg.h>
#include <stdio.h>

#define MAX_PROPERTIES 15

/////////////////////////////////////////////////////////////////////////////
// Storage for the global data in the DLL

#pragma data_seg(".shared")
HWND hNotifyWnd = NULL;

 // Handles to the Shell hooks
HHOOK hKeyboardProcHook = NULL;
HHOOK hLowLevelKeyboardProcHook = NULL;
HHOOK hMouseProcHook = NULL;
HHOOK hLowLevelMouseProcHook = NULL;
HHOOK hGetMsgProcHook = NULL;

HINSTANCE hShellHookInstance = NULL;

long s_prop[MAX_PROPERTIES] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ,0 ,0};

FILE *log_file = NULL;

#pragma data_seg()
#pragma comment(linker, "/SECTION:.shared,RWS")   // tell linker: make it shared

/////////////////////////////////////////////////////////////////////////////
// Per-instance DLL variables

HINSTANCE hTheInstance = NULL;							// This instance of the DLL

LRESULT CALLBACK _KeyboardProcHook(int nCode, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _LowLevelKeyboardProcHook(int nCode, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _MouseProcHook(int nCode, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _LowLevelMouseProcHook(int nCode, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK _GetMsgProcHook(int nCode, WPARAM wParam, LPARAM lParam);

int log(char *fmt, ...);

// The DLL's main procedure
BOOL APIENTRY DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved) {
	// Find out why we're being called
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
		// Save the instance handle
		hTheInstance = (HINSTANCE)hInst;
		// ALWAYS return TRUE to avoid breaking unhookable applications!!!
		return TRUE;
	case DLL_PROCESS_DETACH:
	    //UnSetShellHook();
		return TRUE;

	default:
		return TRUE;
	}
}


// Add the new hook
BOOL _setShellHook(HINSTANCE hInstance) {
    // DLL Not initialized
    if (hInstance == NULL) {
    	return FALSE;
    }

	// Add the ShellProc hook
	hKeyboardProcHook = SetWindowsHookEx(WH_KEYBOARD, (HOOKPROC) _KeyboardProcHook, hInstance, 0L);
    //This makes big problems on w98
    //hMouseProcHook = SetWindowsHookEx(WH_MOUSE, (HOOKPROC) _MouseProcHook, hInstance, 0L);

	hLowLevelKeyboardProcHook = SetWindowsHookEx(WH_KEYBOARD_LL, (HOOKPROC) _LowLevelKeyboardProcHook, hInstance, 0L);
    hLowLevelMouseProcHook = SetWindowsHookEx(WH_MOUSE_LL, (HOOKPROC) _LowLevelMouseProcHook, hInstance, 0L);

	//hGetMsgProcHook = SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC)_GetMsgProcHook, hInstance, 0L);

    //log("hMouseProcHook: %p", hMouseProcHook);

	// Check that it worked
	if ((hKeyboardProcHook != NULL) ||
	    (hMouseProcHook != NULL) ||
	    (hLowLevelKeyboardProcHook != NULL) ||
    (hLowLevelMouseProcHook != NULL)) {

		hShellHookInstance = hInstance;

        PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_UPDATE_STATUS);
		return TRUE;
	}
	// The hook failed, so return an error code
	return FALSE;
}

DllExport BOOL SetShellHook() {
    // DLL Not initialized
    if (hTheInstance == NULL) {
        return FALSE;
    }

	// Don't add a hook if there is already one added
	if ((hKeyboardProcHook != NULL) ||
	    (hMouseProcHook != NULL) ||
	    (hLowLevelKeyboardProcHook != NULL) ||
	    (hLowLevelMouseProcHook != NULL)) {
		return TRUE;
	}

    // Add the ShellProc hook
    if (_setShellHook(hTheInstance)) {
    	return TRUE;
	}
	// The hook failed, so return an error code
	return FALSE;
}

// Remove the hook from the system
DllExport BOOL UnSetShellHook() {

	BOOL unHooked = TRUE;

	// Unhook the the procees
	if (hKeyboardProcHook != NULL) {
		unHooked = UnhookWindowsHookEx(hKeyboardProcHook);
		hKeyboardProcHook = NULL;
	}

	if (hMouseProcHook != NULL) {
		unHooked = UnhookWindowsHookEx(hMouseProcHook);
		hMouseProcHook = NULL;
	}

    if (hLowLevelKeyboardProcHook != NULL) {
		unHooked = UnhookWindowsHookEx(hLowLevelKeyboardProcHook);
		hLowLevelKeyboardProcHook = NULL;
	}

	if (hLowLevelMouseProcHook != NULL) {
		unHooked = UnhookWindowsHookEx(hLowLevelMouseProcHook);
		hLowLevelMouseProcHook = NULL;
	}

	if (hGetMsgProcHook != NULL) {
		unHooked = UnhookWindowsHookEx(hGetMsgProcHook);
		hGetMsgProcHook = NULL;
	}

	// If we managed to unhook then reset
	if (unHooked) {
		hShellHookInstance = NULL;
		PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_UPDATE_STATUS);
	}
	return unHooked;
}

DllExport BOOL SetNotifyWnd(HWND hWnd) {
    hNotifyWnd = hWnd;
    return TRUE;
}

DllExport HWND GetNotifyWnd() {
	return hNotifyWnd;
}

long prop(int id) {
    if ((id < 0) || (id >= MAX_PROPERTIES)) {
        return 0;
    }
    return s_prop[id];
}

#define HKCU HKEY_CURRENT_USER
// Magic registry key/value for "Remove Task Manager" policy.
//
LPCTSTR KEY_DisableTaskMgr =
	"Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System";
LPCTSTR VAL_DisableTaskMgr = "DisableTaskMgr";

void onLock(BOOL locked) {
	static BOOL SystemScreenSaverEnabled = TRUE;
	static BOOL SystemScreenSaverSaved = FALSE;
	HKEY hk;
	DWORD val=1;

	// on W98 disable Ctrl+Alt+Del and the various task-switching keys.
    SystemParametersInfo(SPI_SETSCREENSAVERRUNNING, locked, NULL, 0);
    // Disables task switching
	SystemParametersInfo(SPI_SETFASTTASKSWITCH, !locked, NULL, 0);

	//BlockInput(locked);

	// in debug set the screensaver timeout to 5 seconds.
    //SystemParametersInfo(SPI_SETSCREENSAVETIMEOUT, 5, NULL, 0);

	//detect if the screen saver is enabled
	if (!SystemScreenSaverSaved) {
		SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0, &SystemScreenSaverEnabled, 0);
		SystemScreenSaverSaved = TRUE;
	}

	//To disable the screen saver setting
	if (locked && (prop(WKL_PROP_SCREEN_SAVER_OFF))) {
		SystemParametersInfo( SPI_SETSCREENSAVEACTIVE, FALSE,
                              0, SPIF_SENDWININICHANGE);
	}
	if (!locked) {
		SystemParametersInfo( SPI_SETSCREENSAVEACTIVE, SystemScreenSaverEnabled,
                              0, SPIF_SENDWININICHANGE);
		SystemScreenSaverSaved = FALSE;
	}

	// task manager (Ctrl+Alt+Del)
	if (1) {
		if (RegOpenKey(HKCU, KEY_DisableTaskMgr,&hk)!=ERROR_SUCCESS)
			RegCreateKey(HKCU, KEY_DisableTaskMgr, &hk);

		if (locked) { // disable TM: set policy = 1
			RegSetValueEx(hk, VAL_DisableTaskMgr, 0, REG_DWORD, (BYTE*)&val, sizeof(val));
		} else { // enable TM: remove policy
			RegDeleteValue(hk, VAL_DisableTaskMgr);
		}
	}

}


DllExport BOOL SetProperties(int id, long value) {
	long pvalue;

    if ((id < 0) || (id >= MAX_PROPERTIES)) {
        return FALSE;
    }

	pvalue = s_prop[id];
	s_prop[id] = value;

	// Validation
	if ((prop(WKL_PROP_LOCKED)) &&
		(prop(WKL_PROP_LOCK_KEYBOARD)) &&
		(prop(WKL_PROP_KEY_UNLOCK) == 0) &&
		(prop(WKL_PROP_LOCK_MOUSE_BTN) || prop(WKL_PROP_LOCK_MOUSE_MOVE))) {
		s_prop[WKL_PROP_LOCKED] = 0;
		s_prop[id] = pvalue;
		return FALSE;
	}
	if (id == WKL_PROP_LOCKED) {
		PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_UPDATE_STATUS);
		onLock(value);
	}

    return TRUE;
}


BOOL isLocked() {
    return prop(WKL_PROP_LOCKED);
}

DllExport BOOL GetIsLocked() {
    return (isLocked() &&
            ((hKeyboardProcHook != NULL) ||
	         (hMouseProcHook != NULL) ||
	         (hLowLevelKeyboardProcHook != NULL) ||
	         (hLowLevelMouseProcHook != NULL)));
}

const
	AltMask = 0x20000000; // bit 29

BOOL kbd_MENU = FALSE;
BOOL kbd_CONTROL = FALSE;
BOOL kbd_SHIFT = FALSE;

// Maps to Borland hotkey
long getHotKey(WPARAM vKey, LPARAM lParam, DWORD useAltMask) {
	const scShift = 0x2000;
	const scCtrl  = 0x4000;
	const scAlt   = 0x8000;
	long hk;

	hk = vKey;

    kbd_MENU = GetAsyncKeyState(VK_MENU)  >> ((sizeof(SHORT) * 8) - 1);
    kbd_CONTROL = GetAsyncKeyState(VK_CONTROL) >> ((sizeof(SHORT) * 8) - 1);
    kbd_SHIFT = GetAsyncKeyState(VK_SHIFT) >> ((sizeof(SHORT) * 8) - 1);

	if (kbd_CONTROL) {
		hk += scCtrl;
	}
	if (kbd_SHIFT) {
		hk += scShift;
	}
    if ((lParam & useAltMask) ||
		(kbd_MENU)) {
		hk += scAlt;
	}

	return hk;
}

// Hook procedure for Shell hook

LRESULT CALLBACK _KeyboardProcHook(int nCode, WPARAM wParam, LPARAM lParam) {
const
	KeyReleasedMask = 0x80000000; // bit 31

	if ((lParam & KeyReleasedMask) == 0 /* WM_KEYDOWN */) {
		long key = getHotKey(wParam, lParam, AltMask);
		if (isLocked()) {
			if (key == prop(WKL_PROP_KEY_UNLOCK)) {
				SetProperties(WKL_PROP_LOCKED, 0);
				return 1;
			}
		} else {
			if (key == prop(WKL_PROP_KEY_LOCK)) {
				SetProperties(WKL_PROP_LOCKED, 1);
				return 1;
			}
		}
	}

	// Do we have to handle this message?
    if ((isLocked()) &&
        (prop(WKL_PROP_LOCK_KEYBOARD)) &&
        ((nCode == HC_ACTION) || (nCode == HC_NOREMOVE))) {
        PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_SHOW_LOCKED);
        if (prop(WKL_PROP_DO_BEEP)) MessageBeep(0);
        return 1;
    }

	// Call the next handler in the chain
    return CallNextHookEx(hKeyboardProcHook, nCode, wParam, lParam);
}

LRESULT CALLBACK _LowLevelKeyboardProcHook(int nCode, WPARAM wParam, LPARAM lParam) {
    BOOL State_Key;
    char *sUp;
    far char sKeyCode[40];
	KBDLLHOOKSTRUCT *pkh = (KBDLLHOOKSTRUCT *) lParam;
	long key = getHotKey(pkh->vkCode, pkh->flags, LLKHF_ALTDOWN);

	sUp = "Up   ";
	if (wParam == WM_KEYDOWN) { sUp = "Down "; }
	if (wParam == WM_SYSKEYDOWN) { sUp = "sDown"; }
	if (wParam == WM_SYSKEYUP) { sUp = "sUp  "; }
	GetKeyNameText(pkh->scanCode << 16, sKeyCode, 40);
    log("key %s Shift:%2i, Alt:%2i, Ctrls:%2i  %x %s", sUp, kbd_SHIFT, kbd_MENU, kbd_CONTROL, pkh->vkCode, sKeyCode);


	if ((wParam == WM_SYSKEYDOWN) || (wParam == WM_KEYDOWN)) {
		if (isLocked()) {
			if (key == prop(WKL_PROP_KEY_UNLOCK)) {
				SetProperties(WKL_PROP_LOCKED, 0);
				return 1;
			}
		} else {
			if (key == prop(WKL_PROP_KEY_LOCK)) {
				SetProperties(WKL_PROP_LOCKED, 1);
				return 1;
			}
		}
	}

	// Do we have to handle this message?
    if (isLocked()) {
		 // Avoid Sticky keys
	    State_Key = ((wParam == WM_KEYUP) || (wParam == WM_SYSKEYUP)) &&
	                 (((!kbd_SHIFT) &&  ((pkh->vkCode == VK_RSHIFT) ||
		                                 (pkh->vkCode == VK_LSHIFT))) ||
		              ((!kbd_MENU)  &&  ((pkh->vkCode == VK_LMENU) ||
		                                 (pkh->vkCode == VK_RMENU))) ||
                      ((!kbd_CONTROL) && ((pkh->vkCode == VK_LCONTROL) ||
		                                  (pkh->vkCode == VK_RCONTROL))));

		if ((!State_Key) &&
		    (prop(WKL_PROP_LOCK_KEYBOARD)) &&
            (nCode == HC_ACTION)) {
            // trap Ctrl+Alt+Del
            if ((pkh->vkCode == VK_DELETE) && kbd_MENU && kbd_CONTROL) {
                PostMessage(hNotifyWnd, WM_USER, key, WKL_CMD_HIDE_TASKMGR);
            } else  if ((wParam == WM_SYSKEYDOWN) || (wParam == WM_KEYDOWN)) {
                PostMessage(hNotifyWnd, WM_USER, key, WKL_CMD_SHOW_LOCKED);
                if (prop(WKL_PROP_DO_BEEP)) MessageBeep(0); // only beep on downstroke if requested
            }

            log("key locked:%x", key);
            return 1;
        } else {
            log("key passed:%x, wParam %u", key, wParam);
        }
    } else {
        log("key notlocked:%x", key);
    }

    // Call the next handler in the chain
    return CallNextHookEx(hLowLevelKeyboardProcHook, nCode, wParam, lParam);
}

BOOL MouseDisabled(WPARAM wParam) {
	switch (wParam) {
		case WM_LBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_RBUTTONDOWN:
		case WM_RBUTTONUP: return prop(WKL_PROP_LOCK_MOUSE_BTN);
		case WM_MOUSEMOVE: return prop(WKL_PROP_LOCK_MOUSE_MOVE);
		case WM_MOUSEWHEEL: return prop(WKL_PROP_LOCK_MOUSE_WHEEL);
		default :
		        log("Mouse WPARAM %u", wParam);
		        return FALSE;
	}
}

LRESULT CALLBACK _MouseProcHook(int nCode, WPARAM wParam, LPARAM lParam) {
	if (isLocked()) {
	    if ((MouseDisabled(wParam)) &&
           ((nCode == HC_ACTION) || (nCode == HC_NOREMOVE))) {
           if (wParam != WM_MOUSEMOVE) {
			  //PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_SHOW_LOCKED);
		   }
		   log("mouse locked: nCode %i, wParam %u", nCode, wParam);
           return 1;
        } else {
           log("mouse passed: nCode %i, wParam %u", nCode, wParam);
        }
    }
    // Call the next handler in the chain
    return CallNextHookEx(hMouseProcHook, nCode, wParam, lParam);
}

LRESULT CALLBACK _LowLevelMouseProcHook(int nCode, WPARAM wParam, LPARAM lParam) {
    if ((isLocked()) &&
        (nCode == HC_ACTION) &&
		(MouseDisabled(wParam))) {
		if (wParam != WM_MOUSEMOVE) {
			PostMessage(hNotifyWnd, WM_USER, 0, WKL_CMD_SHOW_LOCKED);
		}
        return 1;
    }
	// Call the next handler in the chain
    return CallNextHookEx(hLowLevelMouseProcHook, nCode, wParam, lParam);
}

LRESULT CALLBACK _GetMsgProcHook(int nCode, WPARAM wParam, LPARAM lParam) {
	MSG *msg = (MSG *)lParam;
	if ((msg->message == WM_SYSCOMMAND) &&
		(msg->wParam == SC_SCREENSAVE)) {
		// TO-DO Detect screen saver start
	}

    // Call the next handler in the chain
    return CallNextHookEx(hGetMsgProcHook, nCode, wParam, lParam);
}

DllExport BOOL StartLog(char* file_name) {
    if (log_file != NULL) {
        fclose(log_file);
    }
    if (file_name == NULL) {
        return TRUE;
    }
    log_file = fopen(file_name, "a+");
    return (log_file != NULL);
}

int log(char *fmt, ...) {
    va_list	argptr;
    if (log_file == NULL) return 0;

    va_start(argptr, fmt);
    vfprintf(log_file, fmt, argptr);
    va_end(argptr);
    fprintf(log_file,"\n");
    fflush(log_file);
    return 1;
}