{ $RCSfile: Properties.pas,v $ $Revision: 1.4 $ $Date: 2004/03/22 02:24:37 $ $Author: vlads $ }
{                                                                                    }
{ ================================================================================== }
{                                                                                    }
{   This file is part of winKeyLock,  http://winKeyLock.sourceforge.net/             }
{   See readme.txt and license.txt for details.                                      }
{   (c) Copyright 2004 - Vlad Skarzhevskyy & co                                      }
{                                                                                    }
{   This program is free software; you can redistribute it and/or modify it under    }
{  the terms of the GNU General Public License as published by the Free Software     }
{  Foundation; either version 2 of the License, or (at your option) any later        }
{  version.                                                                          }
{                                                                                    }
{   This program is distributed in the hope that it will be useful, but WITHOUT ANY  }
{  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A   }
{  PARTICULAR PURPOSE.  See the GNU General Public License for more details.         }
{                                                                                    }
{  The GNU GPL can be found at:       http://www.gnu.org/copyleft/gpl.html           }
{                                                                                    }
{ ================================================================================== }

unit Properties;

interface

uses
    IniFiles;

const
    {$INCLUDE HookDll\win_mesages.inc }

type
    TUserIniFile = TMemIniFile;

    TLockSettings = class
    private
        function GetasBoolean(idx: integer): boolean;
        procedure SetasBoolean(idx: integer; const Value: boolean);
    public
       values: array[1..WKL_PROP_MAX-1] of longint;
       section: string;
       constructor Create();
       procedure Read(f : TUserIniFile);
       procedure Write(f : TUserIniFile);
       property asBoolean[idx: integer]: boolean read GetasBoolean write SetasBoolean;
    end;

    TLockIniFile = class(TUserIniFile)
    private
        FisFirstRun: boolean;
    public
        LockSettings: TLockSettings;
        FileName: String;
        constructor Create(IniFileName: String);
        destructor Destroy; override;

        procedure Write; virtual;
        procedure Read; virtual;
        property isFirstRun: boolean read FisFirstRun;
    end;

    function IsWinXP : boolean;

var
    prop_names: array[1..WKL_PROP_MAX-1] of string;
    IniFile: TLockIniFile;

implementation

uses
    SysUtils,
    Classes,
    Windows,
    Forms;

function IsWinXP : boolean;
begin
  Result := (Win32Platform  = VER_PLATFORM_WIN32_NT) and
            (((Win32MajorVersion = 5) and (Win32MinorVersion >= 1)) or
            (Win32MajorVersion > 5));
end;

{ TLockSettings }

constructor TLockSettings.Create;
var
    i : integer;
begin
    for i := 1 to WKL_PROP_MAX-1 do
    begin
        values[i] := 0;
    end;

    values[WKL_PROP_LOCK_KEYBOARD]    := 1;
    values[WKL_PROP_LOCK_MOUSE_BTN]   := 1;
    values[WKL_PROP_LOCK_MOUSE_MOVE]  := 1;
    values[WKL_PROP_LOCK_MOUSE_WHEEL] := 1;

    values[WKL_PROP_LOKED_IDX] := 2;
    values[WKL_PROP_UNLOKED_IDX] := 3;
end;

function TLockSettings.GetasBoolean(idx: integer): boolean;
begin
    Result := (values[idx] <> 0);
end;

procedure TLockSettings.SetasBoolean(idx: integer; const Value: boolean);
begin
    if (Value) then
        values[idx] := 1
    else
        values[idx] := 0;
end;

procedure TLockSettings.Read(f: TUserIniFile);
var
    i : integer;
begin
    for i := 1 to WKL_PROP_MAX-1 do
    begin
        values[i] := f.ReadInteger(section, prop_names[i] , values[i]);
    end;
end;

procedure TLockSettings.Write(f: TUserIniFile);
var
    i : integer;
begin
    for i := 1 to WKL_PROP_MAX-1 do
    begin
        if (Length(prop_names[i]) > 0) then
            f.WriteInteger(section, prop_names[i], values[i]);
    end;
end;

{ TLockIniFile }

constructor TLockIniFile.Create(IniFileName: String);
var
    vIniFileName :WideString;
begin
    vIniFileName := ExtractFilePath(IniFileName) + ExtractFileName(IniFileName);
    vIniFileName := ChangeFileExt(IniFileName,'.ini');

    inherited Create(vIniFileName);
    LockSettings := TLockSettings.Create;
    LockSettings.section := 'Config.Lock All';
    FisFirstRun := true;
    FileName := vIniFileName;
end;

destructor TLockIniFile.Destroy;
begin
    LockSettings.Free;
    inherited;
end;

procedure TLockIniFile.Read;
var
    Strings: TStringList;
begin
    Strings := TStringList.Create;
    ReadSection(LockSettings.section, Strings);
    FisFirstRun := (Strings.Count = 0);
    LockSettings.Read(self);
    Strings.Free;
end;

procedure TLockIniFile.Write;
begin
    try
        LockSettings.Write(self);
        if (self is  TMemIniFile) then UpdateFile;
        FisFirstRun := false;
    except
        Application.MessageBox(PAnsiChar('Could not save configuration file:' + sLineBreak
                                          + IniFile.FileName),
                                'winKeyLock Error', MB_OK);
    end;
end;

initialization
begin
    prop_names[WKL_PROP_DO_BEEP]          := 'DO_BEEP'         ;
    prop_names[WKL_PROP_LOCK_KEYBOARD]    := 'LOCK_KEYBOARD'   ;
    prop_names[WKL_PROP_LOCK_MOUSE_BTN]   := 'LOCK_MOUSE_BTN'  ;
    prop_names[WKL_PROP_LOCK_MOUSE_MOVE]  := 'LOCK_MOUSE_MOVE' ;
    prop_names[WKL_PROP_LOCK_MOUSE_WHEEL] := 'LOCK_MOUSE_WHEEL';
    prop_names[WKL_PROP_KEY_LOCK]         := 'KEY_LOCK'        ;
    prop_names[WKL_PROP_KEY_UNLOCK]       := 'KEY_UNLOCK'      ;
    prop_names[WKL_PROP_SCREEN_SAVER_OFF] := 'SCREEN_SAVER_OFF';
    prop_names[WKL_PROP_LOKED_IDX]        := 'LOKED_IMAGE';
    prop_names[WKL_PROP_UNLOKED_IDX]      := 'UNLOKED_IMAGE';
end;

end.
